#include <Misc/CloneFactory.h>

#include "GSBonus.h"
#include "GSGame.h"
#include "GDMissileCommand.h"




IMPLEMENT_CLONEABLE( CGSBonus, "GSBonus" )



void CGSBonus::Init()
{
  m_StateDelay  = 0;
  m_State       = BonusState::DELAY_1;

  m_CountedCities   = 0;
  m_CountedMissiles = 0;
  m_SkipBonus       = false;
}



void CGSBonus::Exit()
{
  // TODO - GUIScreen entfernen
}



void CGSBonus::UpdatePerDisplayFrame( const GR::f32 ElapsedTime )
{
  if ( theApp.InputClass()->ReleasedKeyPressed( Xtreme::KEY_ESCAPE ) )
  {
    // set quit flag to auto-add all bonus in the next call to Update
    m_SkipBonus = true;
  }
}



void CGSBonus::RenderCentered( XRenderer& Renderer, XFont* pFont, int Y, const std::string& Text, GR::u32 Color )
{
  int   textWidth = pFont->TextLength( Text.c_str() );
  Renderer.RenderText2d( pFont, ( Renderer.Width() - 2 * textWidth ) / 2, 100, "BONUS", 2.0f, 2.0f, 0xffffffff );
}



void CGSBonus::Display( XRenderer& Renderer )
{
  Renderer.SetShader( XRenderer::ST_ALPHA_TEST );

  auto pFont = theApp.Font( "GUI" );

  RenderCentered( Renderer, pFont, 100, "BONUS", 0xffffffff );

  if ( m_State >= BonusState::COUNT_MISSILES )
  {
    Renderer.RenderText2d( pFont, 200, 160, "UNUSED MISSILES:", 2.0f, 2.0f, 0xffffffff );
    Renderer.RenderText2d( pFont, 500, 160, ( Misc::Format( "%1% x 10 = %2% " ) << m_CountedMissiles << m_CountedMissiles * 10 ).Result().c_str(), 2.0f, 2.0f, 0xffffffff );

    for ( int i = 0; i < m_CountedMissiles; ++i )
    {
      Renderer.RenderTextureSection2d( 300 + i * 10, 210, theApp.Section( "Player.Missile" ) );
    }
  }
  if ( m_State >= BonusState::COUNT_CITIES )
  {
    Renderer.RenderText2d( pFont, 200, 240, "SURVIVING CITIES:", 2.0f, 2.0f, 0xffffffff );
    Renderer.RenderText2d( pFont, 500, 240, ( Misc::Format( "%1% x 200 = %2% " ) << m_CountedCities << m_CountedCities * 200 ).Result().c_str(), 2.0f, 2.0f, 0xffffffff );

    for ( int i = 0; i < m_CountedCities; ++i )
    {
      Renderer.RenderTextureSection2d( 190 + i * 75, 280, theApp.Section( "City" ) );
    }
  }
  if ( m_State >= BonusState::COUNT_CANNONS )
  {
    Renderer.RenderText2d( pFont, 200, 360, "CANNONS LEFT:", 2.0f, 2.0f, 0xffffffff );
    Renderer.RenderText2d( pFont, 500, 360, ( Misc::Format( "%1% x 200 = %2% " ) << m_CountedCannons << m_CountedCannons * 250 ).Result().c_str(), 2.0f, 2.0f, 0xffffffff );

    for ( int i = 0; i < m_CountedCannons; ++i )
    {
      Renderer.RenderTextureSection2d( 400 - 75 - 26 + i * 75, 390, theApp.Section( "Cannon" ) );
    }
  }
}



void CGSBonus::Update()
{
  auto pGSGame = (CGSGame*)m_paramInit.GetParamIP( "GSGame" );

  bool    keepLooping = m_SkipBonus;

  do
  {
    switch ( m_State )
    {
      case BonusState::DELAY_1:
        ++m_StateDelay;
        if ( m_StateDelay >= 50 )
        {
          m_State = BonusState::COUNT_MISSILES;
          m_StateDelay = 0;
        }
        break;
      case BonusState::COUNT_MISSILES:
        ++m_StateDelay;
        if ( m_StateDelay >= 10 )
        {
          bool    hadMissile = false;
          for ( size_t i = 0; i < pGSGame->m_Level.m_Cannons.size(); ++i )
          {
            if ( pGSGame->m_Level.m_Cannons[i].m_Missiles > 0 )
            {
              --pGSGame->m_Level.m_Cannons[i].m_Missiles;
              pGSGame->m_Score += 10;
              hadMissile = true;
              ++m_CountedMissiles;

              theApp.SoundClass()->Play( theApp.Sound( "Blong" ) );
              break;
            }
          }
          m_StateDelay = 0;
          if ( !hadMissile )
          {
            m_State = BonusState::DELAY_2;
          }
        }
        break;
      case BonusState::DELAY_2:
        ++m_StateDelay;
        if ( m_StateDelay >= 40 )
        {
          m_State = BonusState::COUNT_CITIES;
          m_StateDelay = 0;

          m_UnitIterator = pGSGame->m_Level.m_Units.begin();
        }
        break;
      case BonusState::COUNT_CITIES:
        ++m_StateDelay;
        if ( m_StateDelay >= 10 )
        {
          bool    hadCity = false;

          while ( m_UnitIterator != pGSGame->m_Level.m_Units.end() )
          {
            Unit&  unit( *m_UnitIterator );
            ++m_UnitIterator;

            if ( unit.m_Template == "City" )
            {
              pGSGame->m_Score += 200;
              hadCity = true;
              ++m_CountedCities;

              theApp.SoundClass()->Play( theApp.Sound( "Blong" ) );
              break;
            }
          }
          m_StateDelay = 0;
          if ( !hadCity )
          {
            m_State = BonusState::DELAY_3;
          }
        }
        break;
      case BonusState::DELAY_3:
        ++m_StateDelay;
        if ( m_StateDelay >= 40 )
        {
          m_State = BonusState::COUNT_CANNONS;
          m_StateDelay = 0;
          m_UnitIterator = pGSGame->m_Level.m_Units.begin();
        }
        break;
      case BonusState::COUNT_CANNONS:
        ++m_StateDelay;
        if ( m_StateDelay >= 10 )
        {
          bool    hadCannon = false;

          while ( m_UnitIterator != pGSGame->m_Level.m_Units.end() )
          {
            Unit&  unit( *m_UnitIterator );

            ++m_UnitIterator;

            if ( unit.m_Template == "Cannon" )
            {
              pGSGame->m_Score += 250;
              hadCannon = true;
              ++m_CountedCannons;

              theApp.SoundClass()->Play( theApp.Sound( "Blong" ) );
              break;
            }
          }
          m_StateDelay = 0;
          if ( !hadCannon )
          {
            m_State = BonusState::DELAY_4;
          }
        }
        break;
      case BonusState::DELAY_4:
        ++m_StateDelay;
        if ( m_StateDelay >= 50 )
        {
          PopState();
          keepLooping = false;
        }
        break;
    }
  }
  while ( keepLooping );
}



bool CGSBonus::ProcessEvent( const GUI::OutputEvent& Event )
{
  switch ( Event.Type )
  {
    case OET_BUTTON_PUSHED:
      break;
  }
  return false;
}
