#include <Misc/CloneFactory.h>

#include "GSGame.h"
#include "GDMissileCommand.h"
#include "PlayerMissileController.h"
#include "MCTypes.h"




IMPLEMENT_CLONEABLE( CGSGame, "GSGame" )



void CGSGame::Init()
{
  m_Score             = 0;
  m_WaveNumber        = 1;
  m_Cannons           = 3;
  m_GameOverDelay     = 0;
  m_WaitingOnBonusToFinish = false;
  m_WaitForLevelToClear = false;
  m_AllCannonsDestroyed = false;

  GameLogic::_GameEventHandler = fastdelegate::MakeDelegate( this, &CGSGame::OnGameEvent );

  m_Level.m_Cannons.push_back( Cannon( 75, 560, 10 ) );
  m_Level.m_Cannons.push_back( Cannon( 400, 560, 10 ) );
  m_Level.m_Cannons.push_back( Cannon( 725, 560, 10 ) );

  int     cityOffset = ( 400 - 75 ) / 4;

  m_Level.SpawnUnit( "Cannon", 75, 530, 0 );
  m_Level.SpawnUnit( "Cannon", 400, 530, 1 );
  m_Level.SpawnUnit( "Cannon", 725, 530, 2 );

  m_Level.SpawnUnit( "City", 75 + 1 * cityOffset, 530, 0 );
  m_Level.SpawnUnit( "City", 75 + 2 * cityOffset, 530, 1 );
  m_Level.SpawnUnit( "City", 75 + 3 * cityOffset, 530, 2 );
  m_Level.SpawnUnit( "City", 400 + 1 * cityOffset, 530, 3 );
  m_Level.SpawnUnit( "City", 400 + 2 * cityOffset, 530, 4 );
  m_Level.SpawnUnit( "City", 400 + 3 * cityOffset, 530, 5 );

  GameLogic::PrepareWave( m_Level, m_WaveNumber, m_WaveInfo );
}



void CGSGame::Exit()
{
  theApp.SoundClass()->StopAll();
}



void CGSGame::OnGameEvent( GameLogic::EventType::Value Type, Unit* pUnit, Unit* pOtherUnit, int Value )
{
  switch ( Type )
  {
    case GameLogic::EventType::ET_ENEMY_KILLED:
      {
        pUnit->m_RemoveMe = true;

        // TODO - add smaller explosions
        m_Level.SpawnUnit( "Explosion", pUnit->m_Pos.x, pUnit->m_Pos.y );

        GameLogic::RaiseGameEvent( GameLogic::EventType::ET_SCORE_CHANGED, 50 );
      }
      break;
    case GameLogic::EventType::ET_SCORE_CHANGED:
      m_Score += Value;
      break;
    case GameLogic::EventType::ET_CANNON_DESTROYED:
      --m_Cannons;
      if ( m_Cannons <= 0 )
      {
        m_AllCannonsDestroyed = true;
        m_GameOverDelay = 0;
      }
      break;
  }
}



void CGSGame::UpdatePerDisplayFrame( const GR::f32 ElapsedTime )
{
  if ( IsPaused() )
  {
    return;
  }

  if ( m_WaitForLevelToClear )
  {
    if ( m_Level.IsClear() )
    {
      m_WaitForLevelToClear = false;
      m_WaitingOnBonusToFinish = true;


      GR::Strings::ParameterList    paramList;

      paramList.SetParamIP( "CannonsLeft", m_Level.NumberOfCannonsLeft() );
      paramList.SetParamIP( "CitiesLeft", m_Level.NumberOfCitiesLeft() );
      paramList.SetParamIP( "GSGame", (GR::ip)this );
      PushState( "GSBonus", paramList );
      return;
    }
  }

  if ( theApp.InputClass()->ReleasedKeyPressed( Xtreme::KEY_ESCAPE ) )
  {
    GR::Strings::ParameterList    params;

    params.SetParamIP( "InGame", 1 );
    PushState( "GSOptions", params );
    return;
  }
  
  m_Level.Update( ElapsedTime );

  if ( !m_WaitForLevelToClear )
  {
    for ( size_t i = 0; i < m_Level.m_Cannons.size(); ++i )
    {
      if ( theApp.InputClass()->ReleasedVKeyPressed( theApp.BoundKey( MC::ControlKey::FIRE_CANNON_1 + i ) ) )
      {
        if ( m_Level.m_Cannons[i].m_Missiles > 0 )
        {
          --m_Level.m_Cannons[i].m_Missiles;
          ShootCannon( m_Level.m_Cannons[i].m_Pos.x, m_Level.m_Cannons[i].m_Pos.y, theApp.InputClass()->MousePos() );
        }
      }
    }
  }
}



void CGSGame::ShootCannon( int X, int Y, const GR::tPoint& TargetPos )
{
  auto pMissile = m_Level.SpawnUnit( "PlayerMissile", X, Y );

  ( (PlayerMissileController*)pMissile->m_pController )->TargetPos = TargetPos;

  GR::f32     distance = TargetPos.distance( GR::tPoint( X, Y ) );

  ( (PlayerMissileController*)pMissile->m_pController )->StartPos = GR::tPoint( X, Y );
  ( (PlayerMissileController*)pMissile->m_pController )->It.SetValues( 0.0f, 1.0f, distance / 200.0f );

  theApp.SoundClass()->Play( theApp.Sound( Misc::Format( "Missile.Launch.%1%" ) << 1 + rand() % 2 ) );
}



void CGSGame::Display( XRenderer& Renderer )
{
  Renderer.SetShader( XRenderer::ST_FLAT_NO_TEXTURE );
  Renderer.RenderQuad2d( 0, 0, 800, 530, 0xff1FE2FF, 0xff1FE2FF, 0xff4AB1BF, 0xff4AB1BF );
  Renderer.RenderQuad2d( 0, 530, 800, 70, 0xff40c040 );

  Renderer.SetShader( XRenderer::ST_ALPHA_TEST );
  m_Level.Display( Renderer );

  Renderer.SetShader( XRenderer::ST_ALPHA_TEST );
  Renderer.RenderText2d( theApp.Font( "GUI" ), 5, 2, ( Misc::Format( "SCORE %01:6%" ) << m_Score ).Result().c_str(), 2.0f, 2.0f, 0xffffffff );

  std::string     wave = Misc::Format( "WAVE %1%" ) << m_WaveNumber;
  int     textLength = theApp.Font( "GUI" )->TextLength( wave.c_str() );
  Renderer.RenderText2d( theApp.Font( "GUI" ), 800 - 5 - textLength * 2, 2, wave.c_str(), 2.0f, 2.0f, 0xffffffff );

  std::string     hiScore = Misc::Format( "HI %01:6%" ) << theApp.m_BestScore;
  textLength = theApp.Font( "GUI" )->TextLength( hiScore.c_str() );
  Renderer.RenderText2d( theApp.Font( "GUI" ), ( 800 - textLength * 2 ) / 2, 2, hiScore.c_str(), 2.0f, 2.0f, 0xffffffff );

  if ( m_AllCannonsDestroyed )
  {
    textLength = theApp.Font( "GUI" )->TextLength( "GAME OVER" );
    Renderer.RenderText2d( theApp.Font( "GUI" ), ( 800 - textLength * 2 ) / 2, 290, "GAME OVER", 2.0f, 2.0f, 0xffffffff );
  }
}



void CGSGame::Update()
{
  if ( m_WaitForLevelToClear )
  {
    return;
  }
  if ( m_AllCannonsDestroyed )
  {
    ++m_GameOverDelay;
    if ( m_GameOverDelay >= 200 )
    {
      theApp.m_LastScore = m_Score;
      if ( m_Score > theApp.m_BestScore )
      {
        theApp.m_BestScore = m_Score;
      }
      ChangeState( "GSMenu" );
    }
    return;
  }
  m_WaveInfo.Update( m_Level );

  if ( ( m_WaveInfo.m_MissileWaves.empty() )
  &&   ( m_Level.IsWaveComplete() ) )
  {
    m_WaitForLevelToClear = true;
  }
}



bool CGSGame::ProcessEvent( const GUI::OutputEvent& Event )
{
  // TODO - Auf GUI-Events reagieren
  switch ( Event.Type )
  {
    case OET_BUTTON_PUSHED:
      break;
  }
  return false;
}



void CGSGame::OnResume()
{
  if ( m_WaitingOnBonusToFinish )
  {
    m_WaitingOnBonusToFinish = false;
    GameLogic::PrepareWave( m_Level, ++m_WaveNumber, m_WaveInfo );
  }
}