#include "Unit.h"
#include "Level.h"
#include "GDMissileCommand.h"
#include "GameLogic.h"
#include "WaveInfo.h"



namespace GameLogic
{
  tGameEventHandler      _GameEventHandler;


  void RaiseGameEvent( EventType::Value Type, Unit* pUnit, Unit* pOtherUnit, int Value )
  {
    if ( _GameEventHandler )
    {
      _GameEventHandler( Type, pUnit, pOtherUnit, Value );
    }
  }



  void RaiseGameEvent( EventType::Value Type, int Value )
  {
    if ( _GameEventHandler )
    {
      _GameEventHandler( Type, NULL, NULL, Value );
    }
  }



  void PrepareWave( Level& Level, int WaveNo, WaveInfo& Wave )
  {
    Wave.Clear();
    Level.RemoveTempUnits();

    int   numPlayerMissiles = 10;

    int   numMissilesToSpawn = 5 + WaveNo;
    if ( numMissilesToSpawn > 10 )
    {
      numMissilesToSpawn = 10;
    }

    int     numWaves = 1 + WaveNo / 3;
    if ( numWaves > 3 )
    {
      numWaves = 3;
    }
    int numMissilesInWave = 1 + numMissilesToSpawn / numWaves;

    Level::tListUnits::iterator   itC( Level.m_Units.begin() );
    while ( itC != Level.m_Units.end() )
    {
      if ( itC->m_Template == "Cannon" )
      {
        // not destroyed yet
        Level.m_Cannons[itC->m_UnitIndex].m_Missiles = numPlayerMissiles;
      }
      ++itC;
    }

    for ( int i = 0; i < numWaves; ++i )
    {
      Wave.m_MissileWaves[i * 100] = numMissilesInWave;
    }

    Wave.m_MissileSpeed = 80.0f + 4.0f * ( WaveNo - 1 );

    /*
    // TODO - per wave! 
    for ( int i = 0; i < numMissilesToSpawn; ++i )
    {
      Level.SpawnMissile();
    }*/
    theApp.SoundClass()->Play( theApp.Sound( "Sirene" ) );
  }



  void MissileHits( Level& Level, const GR::tPoint& Pos )
  {
    Level::tListUnits::iterator    it2( Level.m_Units.begin() );
    while ( it2 != Level.m_Units.end() )
    {
      Unit&  unit2( *it2 );

      if ( ( unit2.m_Template == "City" )
      ||   ( unit2.m_Template == "Cannon" ) )
      {
        if ( unit2.CollisionBounds().contains( Pos ) )
        {
          Level.SpawnUnit( "Explosion", unit2.m_Pos.x, unit2.m_Pos.y );

          unit2.m_RemoveMe = true;
          if ( unit2.m_Template == "City" )
          {
            Level.SpawnUnit( "City.Destroyed", unit2.m_Pos.x, unit2.m_Pos.y, unit2.m_UnitIndex );
          }
          else if ( unit2.m_Template == "Cannon" )
          {
            Level.SpawnUnit( "Cannon.Destroyed", unit2.m_Pos.x, unit2.m_Pos.y, unit2.m_UnitIndex );

            // remove all missiles
            Level.m_Cannons[unit2.m_UnitIndex].m_Missiles = 0;

            RaiseGameEvent( GameLogic::EventType::ET_CANNON_DESTROYED, unit2.m_UnitIndex );
          }
        }
      }

      ++it2;
    }
  }

}
