#include <Xtreme/X2dRenderer.h>

#include <MasterFrame/XAct.h>

#include <IO/FileChunk.h>

#include <math/MathUtil.h>

#include "level.h"
#include "Unit.h"
#include "GDMissileCommand.h"
#include "UnitController.h"
#include "MissileController.h"
#include "PlayerMissileController.h"
#include "ExplosionController.h"
#include "HurtingExplosionController.h"

#include "GameLogic.h"



Level::Level() :
  m_FixedTime( 0.0f )
  //m_GameEventHandler( NULL ),
{
}



Level::~Level()
{
  Clear();
}



void Level::Display( XRenderer& Renderer )
{
  tListCannons::iterator    itC( m_Cannons.begin() );
  while ( itC != m_Cannons.end() )
  {
    Cannon&    cannon( *itC );

    cannon.Display( Renderer );

    ++itC;
  }

  tListUnits::iterator    it( m_Units.begin() );
  while ( it != m_Units.end() )
  {
    Unit&    unit( *it );

    unit.Display( Renderer );

    ++it;
  }
}



void Level::UpdateUnits( GR::f32 ElapsedTime )
{
  tListCannons::iterator    itC( m_Cannons.begin() );
  while ( itC != m_Cannons.end() )
  {
    Cannon&    cannon( *itC );

    cannon.Update( *this, ElapsedTime );

    ++itC;
  }


  tListUnits::iterator    it( m_Units.begin() );
  while ( it != m_Units.end() )
  {
    Unit&  unit( *it );

    if ( unit.m_RemoveMe )
    {
      //unit.ProcessEvent( *this, Unit::ET_DIE );
      //m_GameEventHandler( Birdie2::CGameEvent( Birdie2::CGameEvent::GE_UNIT_DELETED, pUnit ) );

      it = m_Units.erase( it );
    }
    else
    {
      unit.Update( *this, ElapsedTime );
      if ( unit.m_pController != NULL )
      {
        unit.m_pController->Update( *this, unit, ElapsedTime );
      }
      ++it;
    }
  }
}



void Level::UpdateUnitsFixed()
{
  tListCannons::iterator    itC( m_Cannons.begin() );
  while ( itC != m_Cannons.end() )
  {
    Cannon&    cannon( *itC );

    cannon.UpdateFixed( *this );

    ++itC;
  }

  tListUnits::iterator    it( m_Units.begin() );
  while ( it != m_Units.end() )
  {
    Unit&  unit( *it );

    if ( unit.m_RemoveMe )
    {
      //unit.ProcessEvent( *this, Unit::ET_DIE );
      //m_GameEventHandler( Birdie2::CGameEvent( Birdie2::CGameEvent::GE_UNIT_DELETED, pUnit ) );

      it = m_Units.erase( it );
    }
    else
    {
      unit.UpdateFixed( *this );
      if ( unit.m_pController != NULL )
      {
        unit.m_pController->UpdateFixed( *this, unit );
      }
      ++it;
    }
  }

  it = m_Units.begin();
  while ( it != m_Units.end() )
  {
    next_object:;

    Unit&  unit( *it );

    if ( unit.m_RemoveMe )
    {
      //unit.ProcessEvent( *this, Unit::ET_DIE );
      //m_GameEventHandler( Birdie2::CGameEvent( Birdie2::CGameEvent::GE_UNIT_DELETED, pUnit ) );

      it = m_Units.erase( it );
    }
    else
    {
      GR::tRect   rcBounds( unit.CollisionBounds() );

      tListUnits::iterator    it2( m_Units.begin() );
      while ( it2 != m_Units.end() )
      {
        Unit&  unit2( *it2 );

        if ( ( &unit != &unit2 )
        &&   ( !unit2.m_RemoveMe )
        &&   ( rcBounds.intersects( unit2.CollisionBounds() ) ) )
        {
          GR::u32   dwResult = OnCollision( unit, unit2 );
          if ( dwResult == 2 )
          {
            it2 = m_Units.erase( it2 );
            continue;
          }
          dwResult = OnCollision( unit2, unit );
          if ( dwResult == 2 )
          {
            it = m_Units.erase( it );
            goto next_object;
          }
        }

        ++it2;
      }

      ++it;
    }
  }
}



void Level::Update( const GR::f32 ElapsedTime )
{
  GR::f32     timeLeft = ElapsedTime;

  while ( timeLeft > 0.0f )
  {
    if ( m_FixedTime + timeLeft >= 0.02f )
    {
      GR::f32     timeDelta = 0.02f - m_FixedTime;

      timeLeft -= timeDelta;
      m_FixedTime = 0.0f;

      UpdateUnits( timeDelta );
      UpdateFixed();
    }
    else
    {
      m_FixedTime += timeLeft;
      UpdateUnits( timeLeft );
      timeLeft = 0.0f;
    }
  }
}



void Level::UpdateFixed()
{
  UpdateUnitsFixed();
}



GR::u32 Level::OnCollision( Unit& Unit1, Unit& Unit2 )
{
  //return GameLogic::UnitsCollide( pUnit1, pUnit2, *this );
  return 0;
}



Unit* Level::SpawnUnit( const std::string& Type, int X, int Y, int UnitIndex )
{
  std::string     paramTable = "Unit." + Type;

  m_Units.push_back( Unit() );

  Unit&   unit( m_Units.back() );

  std::string   image = theApp.ParameterList( paramTable, "Image" );
  std::string   anim  = theApp.ParameterList( paramTable, "Animation" );
  
  unit.m_UnitIndex  = UnitIndex;
  unit.m_Template   = Type;
  unit.m_Bounds.set( GR::Convert::ToI32( theApp.ParameterList( paramTable, "X" ) ),
    GR::Convert::ToI32( theApp.ParameterList( paramTable, "Y" ) ),
    GR::Convert::ToI32( theApp.ParameterList( paramTable, "W" ) ),
    GR::Convert::ToI32( theApp.ParameterList( paramTable, "H" ) ) );

  unit.m_LifeTime = GR::Convert::ToI32( theApp.ParameterList( paramTable, "LifeTime" ) );

  if ( image.empty() )
  {
    unit.SetAnimation( anim );
  }
  else
  {
    unit.SetImage( image );
  }

  unit.m_Pos.set( X, Y );
  //unit.m_Direction.set( DX, DY );

  if ( Type == "Missile" )
  {
    unit.m_pController = new MissileController();
  }
  else if ( Type == "PlayerMissile" )
  {
    unit.m_pController = new PlayerMissileController();
  }
  else if ( Type == "Explosion" )
  {
    unit.m_pController = new ExplosionController();
  }
  else if ( Type == "HurtingExplosion" )
  {
    unit.m_pController = new HurtingExplosionController();
  }

  //unit.m_Speed = unit.m_Direction.length();
  return &unit;
}



bool Level::IsBlocked( Unit& MovingUnit, int DX, int DY, Unit*& pBlockingUnit )
{
  GR::tRect   bounds( MovingUnit.CollisionBounds() );

  bounds.offset( DX, DY );

  pBlockingUnit = NULL;

  std::list<Unit>::iterator   itU( m_Units.begin() );
  while ( itU != m_Units.end() )
  {
    Unit&   unit( *itU );

    if ( &unit == &MovingUnit )
    {
      ++itU;
      continue;
    }
    if ( unit.m_Template == "Paddle" )
    {
      if ( bounds.intersects( unit.CollisionBounds() ) )
      {
        pBlockingUnit = &unit;
        return true;
      }
    }

    ++itU;
  }
  return false;
}



bool Level::MoveUnit( Unit& MovingUnit, const GR::tFPoint& Delta )
{
  MovingUnit.m_FractPos += Delta;

  bool blocked = false;

  Unit*   pBlockingUnit = NULL;

  while ( ( MovingUnit.m_FractPos.x < 0.0f )
  ||      ( MovingUnit.m_FractPos.x >= 1.0f )
  ||      ( MovingUnit.m_FractPos.y < 0.0f )
  ||      ( MovingUnit.m_FractPos.y >= 1.0f ) )
  {
    if ( MovingUnit.m_FractPos.x < 0.0f )
    {
      if ( IsBlocked( MovingUnit, -1, 0, pBlockingUnit ) )
      {
        blocked = true;
        MovingUnit.m_FractPos.x = 0.0f;
        if ( ( MovingUnit.m_pController )
        &&   ( pBlockingUnit ) )
        {
          MovingUnit.m_pController->OnBlockedByUnit( MovingUnit, *pBlockingUnit );
        }
      }
      else
      {
        --MovingUnit.m_Pos.x;
        MovingUnit.m_FractPos.x += 1.0f;
      }
    }
    if ( MovingUnit.m_FractPos.x >= 1.0f )
    {
      if ( IsBlocked( MovingUnit, 1, 0, pBlockingUnit ) )
      {
        blocked = true;
        MovingUnit.m_FractPos.x = 0.0f;
        if ( ( MovingUnit.m_pController )
        &&   ( pBlockingUnit ) )
        {
          MovingUnit.m_pController->OnBlockedByUnit( MovingUnit, *pBlockingUnit );
        }
      }
      else
      {
        ++MovingUnit.m_Pos.x;
        MovingUnit.m_FractPos.x -= 1.0f;
      }
    }
    if ( MovingUnit.m_FractPos.y < 0.0f )
    {
      if ( IsBlocked( MovingUnit, 0, -1, pBlockingUnit ) )
      {
        blocked = true;
        MovingUnit.m_FractPos.y = 0.0f;
        if ( ( MovingUnit.m_pController )
        &&   ( pBlockingUnit ) )
        {
          MovingUnit.m_pController->OnBlockedByUnit( MovingUnit, *pBlockingUnit );
        }
      }
      else
      {
        --MovingUnit.m_Pos.y;
        MovingUnit.m_FractPos.y += 1.0f;
      }
    }
    if ( MovingUnit.m_FractPos.y >= 1.0f )
    {
      if ( IsBlocked( MovingUnit, 0, 1, pBlockingUnit ) )
      {
        blocked = true;
        MovingUnit.m_FractPos.y = 0.0f;
        if ( ( MovingUnit.m_pController )
        &&   ( pBlockingUnit ) )
        {
          MovingUnit.m_pController->OnBlockedByUnit( MovingUnit, *pBlockingUnit );
        }
      }
      else
      {
        ++MovingUnit.m_Pos.y;
        MovingUnit.m_FractPos.y -= 1.0f;
      }
    }
  }
  return true;
}



bool Level::UnitIsExplodeable( Unit& Unit )
{
  if ( Unit.m_Template == "Missile" )
  {
    return true;
  }
  return false;
}



void Level::ExplodeObjects( const GR::tPoint& ExplosionCenter, GR::f32 ExplosionRadius )
{
  std::list<Unit>::iterator   itU( m_Units.begin() );
  while ( itU != m_Units.end() )
  {
    Unit&   unit( *itU );

    if ( ( !unit.m_RemoveMe )
    &&   ( UnitIsExplodeable( unit ) ) )
    {
      GR::tRect   bounds( unit.CollisionBounds() );

      //dh::Log( "Circle radius %.2f", ExplosionRadius );

      if ( math::IntersectCircleWithBox2d( GR::tFPoint( (GR::f32)ExplosionCenter.x, (GR::f32)ExplosionCenter.y ), 
                                           ExplosionRadius, 
                                           GR::tFPoint( (GR::f32)bounds.Left, (GR::f32)bounds.Top ),
                                           GR::tFPoint( (GR::f32)bounds.Left + bounds.width(), (GR::f32)bounds.Top + bounds.height() ) ) )
      {
        GameLogic::RaiseGameEvent( GameLogic::EventType::ET_ENEMY_KILLED, &unit );
      }
    }

    ++itU;
  }
}



void Level::SpawnMissile( GR::f32 MissileSpeed )
{
  GR::f32           angle = 0.0f;

  GR::tPoint        startPos;
  GR::tFPoint       movePath;

  // loop until the missile appears on screen

  std::vector<int>    targetPos;
  int     cityOffset = ( 400 - 75 ) / 4;

  targetPos.push_back( 75 );
  targetPos.push_back( 400 );
  targetPos.push_back( 725 );
  targetPos.push_back( 75 + 1 * cityOffset );
  targetPos.push_back( 75 + 2 * cityOffset );
  targetPos.push_back( 75 + 3 * cityOffset );
  targetPos.push_back( 400 + 1 * cityOffset );
  targetPos.push_back( 400 + 2 * cityOffset );
  targetPos.push_back( 400 + 3 * cityOffset );

  do
  {
    int     targetCity = rand() % targetPos.size();

    int     targetX = targetPos[targetCity];
    int     targetY = 550;

    angle = rand() % 136 + 180 + 22.5f;

    GR::tPoint       endPos( targetX, targetY );

    movePath.set( 800.0f + 10.0f * ( rand() % 20 ), 0.0f );
    movePath.RotateZ( angle );

    startPos.set( (int)( targetX - movePath.x ), (int)( targetY + movePath.y ) );
  }
  while ( ( startPos.x < 0 )
  ||      ( startPos.x >= 800 ) );


  Unit*   pMissile = SpawnUnit( "Missile", startPos.x, startPos.y );
  pMissile->m_DisplayAngle = angle - 90.0f;
  pMissile->m_Speed = MissileSpeed * movePath.normalized();
  pMissile->m_Speed.y = -pMissile->m_Speed.y;
}



bool Level::IsWaveComplete()
{
  std::list<Unit>::iterator   itU( m_Units.begin() );
  while ( itU != m_Units.end() )
  {
    Unit&   unit( *itU );

    if ( unit.m_Template == "Missile" )
    {
      return false;
    }

    ++itU;
  }
  return true;
}



bool Level::IsClear()
{
  std::list<Unit>::iterator   itU( m_Units.begin() );
  while ( itU != m_Units.end() )
  {
    Unit&   unit( *itU );

    if ( ( unit.m_Template == "Explosion" )
    ||   ( unit.m_Template == "HurtingExplosion" )
    ||   ( unit.m_Template == "Player.Missile" ) )
    {
      return false;
    }

    ++itU;
  }
  return true;
}



void Level::RemoveTempUnits()
{
  std::list<Unit>::iterator   itU( m_Units.begin() );
  while ( itU != m_Units.end() )
  {
    Unit&   unit( *itU );

    if ( ( unit.m_Template != "City" )
    &&   ( unit.m_Template != "City.Destroyed" )
    &&   ( unit.m_Template != "Cannon" )
    &&   ( unit.m_Template != "Cannon.Destroyed" ) )
    {
      itU = m_Units.erase( itU );
    }
    else
    {
      ++itU;
    }
  }
}



void Level::Clear()
{
  m_Units.clear();
  m_Cannons.clear();
}



int Level::NumberOfCannonsLeft()
{
  return (int)m_Cannons.size();
}



int Level::NumberOfCitiesLeft()
{
  int     numCities = 0;

  std::list<Unit>::iterator   itU( m_Units.begin() );
  while ( itU != m_Units.end() )
  {
    Unit&   unit( *itU );

    if ( unit.m_Template == "City" )
    {
      ++numCities;
    }
    ++itU;
  }
  return numCities;
}
