#include "Unit.h"
#include "Level.h"
#include "GDMissileCommand.h"
#include "UnitController.h"

#include <Xtreme/XRenderer.h>



Unit::Unit() :
  m_pController( NULL ),
  m_RemoveMe( false ),
  m_DisplayAngle( 0.0f ),
  m_LifeTime( 0 ),
  m_DisplayZoomFactor( 1.0f ),
  m_UnitIndex( 0 )
{
}



Unit::~Unit()
{
  delete m_pController;
}



void Unit::Display( XRenderer& Renderer )
{
  XTextureSection   ts;

  if ( !m_Image.empty() )
  {
    ts = theApp.Section( m_Image );
  }
  else
  {
    ts = theApp.AnimationFrame( m_AnimPos );
  }

  /*
  if ( m_DisplayAngle == 0.0f )
  {
    Renderer.RenderTextureSection2d( m_Pos.x - ts.m_Width / 2, m_Pos.y - ts.m_Height / 2, ts );
  }
  else*/
  {
    GR::tFPoint     tuv[4];

    ts.GetTrueUV( tuv[0], tuv[1], tuv[2], tuv[3] );

    int       width = ts.m_Width;
    int       height = ts.m_Height;

    int       x = m_Pos.x;
    int       y = m_Pos.y;

    GR::f32   zoomX = m_DisplayZoomFactor;
    GR::f32   zoomY = m_DisplayZoomFactor;

    GR::tVector   vDelta( -width * 0.5f * zoomX, -height * 0.5f * zoomY, 0.0f );
    GR::tVector   vDelta2( width * 0.5f * zoomX, -height * 0.5f * zoomY, 0.0f );
    GR::tVector   vDelta3( -width * 0.5f * zoomX, height * 0.5f * zoomY, 0.0f );
    GR::tVector   vDelta4( width * 0.5f * zoomX, height * 0.5f * zoomY, 0.0f );

    vDelta.RotateZ( -m_DisplayAngle );
    vDelta2.RotateZ( -m_DisplayAngle );
    vDelta3.RotateZ( -m_DisplayAngle );
    vDelta4.RotateZ( -m_DisplayAngle );

    GR::tVector   ptPos1 = GR::tVector( (float)x, (float)y, 0 ) + vDelta;
    GR::tVector   ptPos2 = GR::tVector( (float)x, (float)y, 0 ) + vDelta2;
    GR::tVector   ptPos3 = GR::tVector( (float)x, (float)y, 0 ) + vDelta3;
    GR::tVector   ptPos4 = GR::tVector( (float)x, (float)y, 0 ) + vDelta4;

    Renderer.SetTexture( 0, ts.m_pTexture );
    Renderer.RenderQuadDetail2d( ptPos1.x, ptPos1.y,
      ptPos2.x, ptPos2.y,
      ptPos3.x, ptPos3.y,
      ptPos4.x, ptPos4.y,
      tuv[0].x, tuv[0].y,
      tuv[1].x, tuv[1].y,
      tuv[2].x, tuv[2].y,
      tuv[3].x, tuv[3].y,
      0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff );
  }
}



void Unit::UpdateFixed( Level& Level )
{
  if ( m_LifeTime > 0 )
  {
    --m_LifeTime;
    if ( m_LifeTime == 0 )
    {
      m_RemoveMe = true;
    }
  }
}



void Unit::Update( Level& Level, GR::f32 ElapsedTime )
{
  //m_DisplayAngle += 20.0f * ElapsedTime;
  if ( !m_Animation.empty() )
  {
    theApp.m_AnimationManager.AdvanceAnimFrame( m_AnimPos, ElapsedTime * 1000.0f );
  }

  Level.MoveUnit( *this, m_Speed * ElapsedTime );
}



void Unit::SetImage( const std::string& Image )
{
  m_Animation.clear();
  m_Image = Image;
}



void Unit::SetAnimation( const std::string& Animation )
{
  m_Image.clear();
  if ( m_Animation != Animation )
  {
    m_Animation = Animation;

    m_AnimPos = theApp.AnimationPos( m_Animation );
  }
}



GR::tRect Unit::CollisionBounds()
{
  GR::tRect     movedBounds( m_Bounds );
  return movedBounds.offset( m_Pos );
}